@@ -6,10 +6,13 @@ use JMAP::Validation::Tests::ContactGroup;
 use JMAP::Validation::Tests::Object;
 use JMAP::Validation::Tests::SetError;
 use JMAP::Validation::Tests::String;
+use JSON::Typist;
 use Test2::Bundle::Extended;
 
 # record types {{{
 
+my $typist = JSON::Typist->new();
+
 my %ContactGroup_checks = (
   id => $JMAP::Validation::Checks::String::is_id,
 
@@ -36,16 +39,8 @@ our $is_ContactGroup_for_create = hash {
 };
 
 our $is_ContactGroup_for_update = hash{
-  field name => in_set(
-    FDNE(),
-    $ContactGroup_checks{name},
-  );
-
-  field contactIds => in_set(
-    FDNE(),
-    $ContactGroup_checks{contactIds},
-  );
-
+  field name       => in_set($ContactGroup_checks{name}, U());
+  field contactIds => in_set($ContactGroup_checks{contactIds}, U());
   end();
 };
 
@@ -83,33 +78,43 @@ our $getContactGroups_args = hash {
   end();
 };
 
-
 our $setContactGroups_args = hash {
-  field accountId => in_set(
-    FDNE(),
-    $JMAP::Validation::Checks::String::is_string_or_null,
-  );
-
-  field ifInState => in_set(
-    FDNE(),
-    $JMAP::Validation::Checks::String::is_string_or_null,
-  );
+  field accountId => in_set($JMAP::Validation::Checks::String::is_string_or_null, U());
+  field ifInState => in_set($JMAP::Validation::Checks::String::is_string_or_null, U());
 
   field create => in_set(
-    FDNE(),
-    hash{ all_values $is_ContactGroup_for_create },
+    hash{
+      all_keys validator(sub {
+        my %params = @_;
+
+        return JMAP::Validation::validate(
+          $typist->apply_types($params{got}),
+          $JMAP::Validation::Checks::String::is_id,
+        );
+      });
+
+      all_values $is_ContactGroup_for_create;
+    },
+    U(),
   );
 
   field update => in_set(
-    FDNE(),
-    hash{ all_values $is_ContactGroup_for_update },
-  );
+    hash{
+      all_keys validator(sub {
+        my %params = @_;
 
-  field destroy => in_set(
-    FDNE(),
-    array { all_items $JMAP::Validation::Checks::String::is_id },
+        return JMAP::Validation::validate(
+          $typist->apply_types($params{got}),
+          $JMAP::Validation::Checks::String::is_id,
+        );
+      });
+
+      all_values $is_ContactGroup_for_update;
+    },
+    U(),
   );
 
+  field destroy => in_set(array { all_items $JMAP::Validation::Checks::String::is_id }, U());
   end();
 };
 
